import asyncio

import dico


def parse_bytesize(bytesize: float):
    gb = round(bytesize / (1024 * 1024 * 1024), 1)
    if gb < 1:
        mb = round(bytesize / (1024 * 1024), 1)
        if mb < 1:
            return f"{bytesize}KB"
        return f"{mb}MB"
    return f"{gb}GB"


def parse_second(time: int):
    parsed_time = ""
    day = time // (24 * 60 * 60)
    time -= day * (24 * 60 * 60)
    hour = time // (60 * 60)
    time -= hour * (60 * 60)
    minute = time // 60
    time -= minute * 60
    if day:
        parsed_time += f"{day} day{'s' if day != 1 else ''} "
    if hour:
        parsed_time += f"{hour} hours "
    if minute:
        parsed_time += f"{minute} minutes "
    parsed_time += f"{time} seconds"
    return parsed_time


async def delete_wait(bot, ctx, content=None, embed=None):
    delete_button = dico.Button(style=dico.ButtonStyles.DANGER, emoji="🗑️", custom_id="trash")
    delete_button.custom_id += str(ctx.id)
    msg = await ctx.reply(content, embed=embed, component=dico.ActionRow(delete_button))
    try:
        await bot.wait("interaction_create",
                       check=lambda inter: int(inter.author) == int(ctx.author.id) and inter.data.custom_id == delete_button.custom_id,
                       timeout=60)
        await msg.delete()
    except asyncio.TimeoutError:
        delete_button.disabled = True
        await msg.edit(component=dico.ActionRow(delete_button))



import asyncio
import traceback

import dico
import dico_command
from .components import DP_INFO_SELECT, DPE_INFO_SELECT
from .infos import get_front_page, get_bot_info, get_sys_info, get_full_info
from .pager import Pager
from .pyeval import PYEval
from .utils import delete_wait


def interaction_check(message_id, prefix):
    def wrap(inter: dico.Interaction):
        return inter.data.custom_id.startswith(prefix) and inter.data.custom_id.endswith(str(message_id))
    return wrap


class DPAddon(dico_command.Addon, name="dp"):
    command_name = "dp"

    async def addon_check(self, ctx):
        owner_ids = await self.bot.get_owners()
        return ctx.author.id in owner_ids

    @dico_command.command(command_name)
    async def dp(self, ctx: dico_command.Context):
        info = DP_INFO_SELECT.copy()
        info.custom_id += str(ctx.id)
        msg = await ctx.reply(get_front_page(self.bot, ctx), component=dico.ActionRow(info))

        while not self.bot.websocket_closed:
            try:
                interaction: dico.Interaction = await self.bot.wait("interaction_create", check=interaction_check(ctx.id, "dpinfo"), timeout=30)
                await interaction.create_response(dico.InteractionResponse(callback_type=dico.InteractionCallbackType.DEFERRED_UPDATE_MESSAGE, data={}))
                if interaction.data.values[0] == "main":
                    await msg.edit(content=get_front_page(self.bot, ctx))
                elif interaction.data.values[0] == "bot":
                    await msg.edit(content=get_bot_info(self.bot, ctx.guild_id))
                elif interaction.data.values[0] == "sys":
                    await msg.edit(content=get_sys_info())
            except asyncio.TimeoutError:
                break
        info.disabled = True
        await msg.edit(component=dico.ActionRow(info))

    @dp.subcommand("bot")
    async def dp_bot(self, ctx: dico_command.Context):
        await ctx.reply(get_bot_info(self.bot, ctx.guild_id))

    @dp.subcommand("sys")
    async def dp_sys(self, ctx: dico_command.Context):
        await ctx.reply(get_sys_info())

    @dp.subcommand("py")
    async def dp_py(self, ctx: dico_command.Context, *, src: str):
        ev = PYEval(ctx, self.bot, src)
        delete_button = dico.Button(style=dico.ButtonStyles.DANGER, emoji="🗑️", custom_id="trash")
        try:
            resp = await ev.evaluate()
            str_resp = [f"Result {i}:\n{x}" for i, x in enumerate(resp, start=1)] if len(resp) > 1 else resp
            pages = []
            for x in str_resp:
                if len(x) > 2000:
                    while len(x) > 2000:
                        pages.append(x[:2000])
                        x = x[2000:]
                    pages.append(x)
                else:
                    pages.append(x)
            if len(pages) == 1:
                return await delete_wait(self.bot, ctx, content=str(pages[0]))
            pager = Pager(self.bot, ctx.channel, ctx.author, pages, reply=ctx, extra_button=delete_button, timeout=60)
            async for _ in pager.start():
                await pager.message.delete()
                break
        except Exception as ex:
            tb = ''.join(traceback.format_exception(type(ex), ex, ex.__traceback__))
            tb = ("..." + tb[-1997:]) if len(tb) > 2000 else tb
            await delete_wait(self.bot, ctx, content=tb)


class DPEAddon(dico_command.Addon, name="dpe"):
    command_name = "dpe"

    async def addon_check(self, ctx):
        owner_ids = await self.bot.get_owners()
        return ctx.author.id in owner_ids

    @dico_command.command(command_name)
    async def dpe(self, ctx: dico_command.Context):
        info = DPE_INFO_SELECT.copy()
        info.custom_id += str(ctx.id)
        msg = await ctx.reply(embed=get_front_page(self.bot, ctx, embed=True), component=dico.ActionRow(info))

        while not self.bot.websocket_closed:
            try:
                interaction: dico.Interaction = await self.bot.wait("interaction_create", check=interaction_check(ctx.id, "dpeinfo"), timeout=30)
                await interaction.create_response(dico.InteractionResponse(callback_type=dico.InteractionCallbackType.DEFERRED_UPDATE_MESSAGE, data={}))
                if interaction.data.values[0] == "main":
                    await msg.edit(embed=get_front_page(self.bot, ctx, embed=True))
                elif interaction.data.values[0] == "bot":
                    await msg.edit(embed=get_bot_info(self.bot, ctx.guild_id, embed=True, timestamp=ctx.timestamp))
                elif interaction.data.values[0] == "sys":
                    await msg.edit(embed=get_sys_info(embed=True, timestamp=ctx.timestamp))
                elif interaction.data.values[0] == "full":
                    await msg.edit(embed=get_full_info(self.bot, ctx))
            except asyncio.TimeoutError:
                break
        info.disabled = True
        await msg.edit(component=dico.ActionRow(info))

    @dpe.subcommand("bot")
    async def dpe_bot(self, ctx: dico_command.Context):
        await ctx.reply(embed=get_bot_info(self.bot, ctx.guild_id, embed=True, timestamp=ctx.timestamp))

    @dpe.subcommand("sys")
    async def dpe_sys(self, ctx: dico_command.Context):
        await ctx.reply(embed=get_sys_info(embed=True, timestamp=ctx.timestamp))

    @dpe.subcommand("full")
    async def dpe_full(self, ctx: dico_command.Context):
        await ctx.reply(embed=get_full_info(self.bot, ctx))



import typing
import asyncio
import dico
import dico_command


class Pager:
    next_emoji = "➡"
    prev_emoji = "⬅"

    def __init__(self,
                 bot: dico_command.Bot,
                 channel: dico.Channel,
                 author: dico.User,
                 pages: typing.List[typing.Union[str, dico.Embed]],
                 extra_button: dico.Button = None,
                 *, is_embed: bool = False,
                 reply: dico.Message = None,
                 timeout: int = 30):
        self.bot = bot
        self.channel = channel
        self.author = author
        self.pages = pages
        self.extra_button = extra_button
        self.is_embed = is_embed
        self.reply = reply
        self.timeout = timeout
        self.current = 0
        self.message = None  # Should be set later.

    @property
    def __max_page(self):
        return len(self.pages) - 1

    @property
    def current_page(self):
        return self.current + 1

    def next(self):
        self.current = self.current + 1 if self.current + 1 <= self.__max_page else 0
        return self.pages[self.current]

    def prev(self):
        self.current = self.current - 1 if self.current - 1 >= 0 else self.__max_page
        return self.pages[self.current]

    @property
    def page_button(self):
        return dico.Button(style=2, label=f"Page {self.current_page}/{len(self.pages)}", custom_id=f"pages{self.message.id}", disabled=True)

    def current_action_row(self, disabled: bool = False):
        next_button = dico.Button(style=1, label="Next", custom_id=f"next{self.message.id}", emoji=self.next_emoji, disabled=disabled)
        prev_button = dico.Button(style=1, label="Prev", custom_id=f"prev{self.message.id}", emoji=self.prev_emoji, disabled=disabled)
        buttons = [prev_button, self.page_button, next_button]
        if self.extra_button:
            self.extra_button.custom_id += str(self.message.id)
            if disabled:
                self.extra_button.disabled = True
            buttons.append(self.extra_button)
        return dico.ActionRow(*buttons)

    async def start_flatten(self):
        return_list = []
        async for x in self.start():
            return_list.append(x)
        return return_list

    async def start(self):
        func = self.channel.send if not self.reply else self.reply.reply
        self.message = await func(content=self.pages[0] if not self.is_embed else None, embed=self.pages[0] if self.is_embed else None)
        await self.message.edit(component=self.current_action_row())

        while not self.bot.websocket_closed:
            try:
                interaction: dico.Interaction = await self.bot.wait(
                    "interaction_create",
                    check=lambda inter: int(inter.author) == int(self.author.id),
                    timeout=self.timeout
                )

                await interaction.create_response(dico.InteractionResponse(callback_type=dico.InteractionCallbackType.DEFERRED_UPDATE_MESSAGE, data={}))

                if interaction.data.custom_id.startswith("next"):
                    page = self.next()
                    await self.message.edit(content=page if not self.is_embed else None,
                                            embed=page if self.is_embed else None,
                                            component=self.current_action_row())

                elif interaction.data.custom_id.startswith("prev"):
                    page = self.prev()
                    await self.message.edit(content=page if not self.is_embed else None,
                                            embed=page if self.is_embed else None,
                                            component=self.current_action_row())

                else:
                    yield self.current_page

            except asyncio.TimeoutError:
                break
        await self.message.edit(component=self.current_action_row(disabled=True))



import dico


SYS_BUTTON = dico.Button(style=dico.ButtonStyles.DANGER, label="System Info", custom_id="sys")
BOT_BUTTON = dico.Button(style=dico.ButtonStyles.PRIMARY, label="Bot Info", custom_id="bot")

DP_INFO_SELECT = dico.SelectMenu(custom_id="dpinfo",
                                 options=[
                                     dico.SelectOption(label="Main Panel", value="main"),
                                     dico.SelectOption(label="System Info", value="sys"),
                                     dico.SelectOption(label="Bot Info", value="bot")
                                 ])

DPE_INFO_SELECT = dico.SelectMenu(custom_id="dpeinfo",
                                  options=[
                                      dico.SelectOption(label="Main Panel", value="main"),
                                      dico.SelectOption(label="System Info", value="sys"),
                                      dico.SelectOption(label="Bot Info", value="bot"),
                                      dico.SelectOption(label="Full Info", value="full")
                                  ])



"""
dico
~~~~~~~~~~~~~~~~~~~~~~~~
Simple debugger and tester for dico-command.
:copyright: (c) 2021 dico-api
:license: MIT
"""

__version__ = "0.0.1"

from .addon import DPAddon, DPEAddon


def load(bot):
    bot.load_addons(DPAddon, DPEAddon)


def unload(bot):
    bot.unload_addons(DPAddon, DPEAddon)



import setuptools

with open("README.md", "r", encoding="UTF-8") as f:
    long_description = f.read()

setuptools.setup(
    name="dico-dp",
    version="0.0.1",
    author="eunwoo1104",
    author_email="sions04@naver.com",
    description="Simple debugger and tester for dico-command.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dico-api/dico-dp",
    packages=setuptools.find_packages(),
    python_requires='>=3.7',
    install_requires=["dico-api", "dico-command"],
    classifiers=[
        "Programming Language :: Python :: 3"
    ]
)
